@@ -13,6 +13,7 @@ var states = {
     InActivity: "$$.InActivity",
     IsLayoutManagementAvailable: "$$.IsLayoutManagementAvailable",
     IsLayoutManagementSupported: "$$.IsLayoutManagementSupported",
+    IsOpenRemoteFileSupported: "$$.IsOpenRemoteFileSupported",
     HasMixedDPIScalingHandling: "$$.HasMixedDPIScalingHandling",
     HasOpenWindows: "$$.HasOpenWindows",
     HasSamples: "$$.HasSamples",
@@ -93,6 +94,18 @@ addPlugin({
         },
     },
 
+    documents: {
+        fileTypes: {
+            "welcome": {
+                icon: "",
+                factoryName: "WelcomeDocumentFactory",
+                viewFactories: [
+                    { factoryName: "WelcomeDocumentViewFactory", priority: 50 }
+                ]
+            },
+        }
+    },
+
     documentWells: {
         main: {
             closeWhenEmpty: false
@@ -151,6 +164,12 @@ addPlugin({
             shortcut: codeTr("Ctrl+O"),
             importance: "high"
         },
+        "$$.OpenRemoteFile": {
+            text: qsTr("Open &Remote File..."),
+            shortcut: codeTr("Ctrl+Shift+O"),
+            visibleWithFlags: [states.IsOpenRemoteFileSupported],
+            importance: "high"
+        },
         "$$.SaveFile": {
             text: qsTr("&Save File"),
             imageFontIcon: "fa-save",
@@ -344,12 +363,13 @@ addPlugin({
         "$$.FileGroup": {
             "$$.NewFile": { order: 100 },
             "$$.OpenFile": { order: 110 },
-            "$$.SaveFile": { order: 120 },
-            "$$.SaveFileAs": { order: 130 },
-            "$$.SaveAllFiles": { order: 140 },
-            "$$.CloseFile": { order: 150 },
-            "$$.CloseAllFiles": { order: 160 },
-            "$$.RecentFiles" : { order: 170 },
+            "$$.OpenRemoteFile": { order: 120 },
+            "$$.SaveFile": { order: 130 },
+            "$$.SaveFileAs": { order: 140 },
+            "$$.SaveAllFiles": { order: 150 },
+            "$$.CloseFile": { order: 160 },
+            "$$.CloseAllFiles": { order: 170 },
+            "$$.RecentFiles" : { order: 180 },
         },
 
         // NOTE: Battle & Rebel have their own product specific layout group (but should really use this)
@@ -506,7 +526,7 @@ addPlugin({
                     detailsText: qsTr("General font used for all non-code UI elements."),
                     changeRequiresAppRestart: true,
                 },
-                "$$.CodeFont": {
+                "$$.CodeFont2": {
                     order: 102,
                     type: "font",
                     fontType: "code",
@@ -527,8 +547,8 @@ addPlugin({
                     requiresFlags: ["CorePlugin.HasProjectService"],
                     order: 110,
                     type: "directoryPath",
-                    category: qsTr("Default Document Folder"),
-                    detailsText: qsTr("Directory where documents unassociated with a project will be saved."),
+                    category: qsTr("Documents Folder"),
+                    detailsText: qsTr("The folder where projects and documents will be saved."),
                 },
                 "$$.StartupBehavior": {
                     requiresFlags: ["CorePlugin.HasProjectService"],
@@ -682,7 +702,7 @@ if (AppLib.environment.has_enhanced_windowing_experience) {
                         changeRequiresAppRestart: true,
                     },
 
-                    // XXX: This needs some style changes to support; the borders are currently drawn 
+                    // XXX: This needs some style changes to support; the borders are currently drawn
                     // incorrectly.
                     //
                     // "$$.TabBarPosition": {
