@@ -1,4 +1,4 @@
-# Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2023, NVIDIA CORPORATION. All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
 # are met:
@@ -36,6 +36,52 @@ def get_description():
 def get_section_identifier():
     return "MemoryWorkloadAnalysis_Chart"
 
+
+def get_estimated_speedup(action, aperture):
+    all_lookup_misses = action["lts__t_sectors_srcunit_tex_lookup_miss.sum"].value()
+    aperture_lookup_misses = action["lts__t_sectors_srcunit_tex_aperture_{}_lookup_miss.sum".format(aperture)].value()
+
+    dram_bandwidth_name = "dram__bytes.sum.per_second"
+    if dram_bandwidth_name in action:
+        dram_bandwidth = action[dram_bandwidth_name].value()
+    else:
+        dram_bandwidth = 0
+
+    pcie_read_bandwidth_name = "pcie__read_bytes.sum.per_second"
+    pcie_write_bandwidth_name = "pcie__write_bytes.sum.per_second"
+    if (pcie_read_bandwidth_name in action) and (pcie_write_bandwidth_name in action):
+        pcie_bandwidth = action[pcie_read_bandwidth_name].value() + action[pcie_write_bandwidth_name].value()
+    else:
+        pcie_bandwidth = 0
+
+    if aperture == "sysmem":
+        # System memory is expected to be connected via PCIe
+        aperture_bandwidth = pcie_bandwidth
+    elif aperture == "peer":
+        # Peer memory is expected to be connected via PCIe or NVLink
+        nvlink_read_bandwidth_name = "nvlrx__bytes.sum.per_second"
+        nvlink_write_bandwidth_name = "nvltx__bytes.sum.per_second"
+        if (nvlink_read_bandwidth_name in action) and (nvlink_write_bandwidth_name in action):
+            nvlink_bandwidth = action[nvlink_read_bandwidth_name].value() + \
+                action[nvlink_write_bandwidth_name].value()
+        else:
+            nvlink_bandwidth = 0
+        aperture_bandwidth = max(pcie_bandwidth, nvlink_bandwidth)
+    else:
+        # unknown aperture, cannot calculate speedup
+        return NvRules.IFrontend.SpeedupType_LOCAL, 0
+
+    if all_lookup_misses != 0 and dram_bandwidth != 0 and aperture_bandwidth != 0:
+        # Only give an estimate if we could collect some value for the aperture bandwidth
+        improvement_percent = (aperture_lookup_misses / all_lookup_misses) * (1 - aperture_bandwidth / dram_bandwidth) * 100
+        speedup_type = NvRules.IFrontend.SpeedupType_GLOBAL
+    else:
+        improvement_percent = 0
+        speedup_type = NvRules.IFrontend.SpeedupType_LOCAL
+
+    return speedup_type, improvement_percent
+
+
 def apply(handle):
     ctx = NvRules.get_context(handle)
     action = ctx.range_by_idx(0).action_by_idx(0)
@@ -69,7 +115,8 @@ def apply(handle):
 
     for aperture in apertures:
         aperture_info = apertures[aperture]
-        lts__t_sectors_srcunit_tex_aperture_lookup_miss = action.metric_by_name("lts__t_sectors_srcunit_tex_aperture_{}_lookup_miss.sum".format(aperture)).as_double()
+        metric_name = "lts__t_sectors_srcunit_tex_aperture_{}_lookup_miss.sum".format(aperture)
+        lts__t_sectors_srcunit_tex_aperture_lookup_miss = action[metric_name].value()
         lts__t_sectors_srcunit_tex_aperture_lookup_miss_ratio = 100. * lts__t_sectors_srcunit_tex_aperture_lookup_miss / lts__t_sectors_srcunit_tex_lookup_miss if lts__t_sectors_srcunit_tex_lookup_miss else 0.
 
         if lts__t_sectors_srcunit_tex_peak_pct > lts__high_utilization_threshold and lts__t_sectors_srcunit_tex_aperture_lookup_miss_ratio > lts__high_aperture_utilization_threshold:
@@ -80,6 +127,9 @@ def apply(handle):
                 message += " Tweaking the L2 cache policies can help optimizing the cache hit rates for accesses to slower {} memory. ".format(aperture_info.lower())
                 message += "Lookup CUaccessProperty and policy CU_ACCESS_PROPERTY_PERSISTING for more details."
 
-            msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_WARNING, message, "{} Memory Usage".format(aperture_info))
-            fe.focus_metric(msg_id, "Peak % of L2 Sector Requests from L1TEX", lts__t_sectors_srcunit_tex_peak_pct, NvRules.IFrontend.Severity_SEVERITY_HIGH if lts__t_sectors_srcunit_tex_peak_pct > 75 else NvRules.IFrontend.Severity_SEVERITY_LOW, "{:.2f} < {:.2f}".format(lts__high_utilization_threshold, lts__t_sectors_srcunit_tex_peak_pct))
-            fe.focus_metric(msg_id, "% of L2 Misses to {} Memory".format(aperture_info), lts__t_sectors_srcunit_tex_aperture_lookup_miss_ratio, NvRules.IFrontend.Severity_SEVERITY_HIGH if lts__t_sectors_srcunit_tex_aperture_lookup_miss_ratio > 75 else NvRules.IFrontend.Severity_SEVERITY_LOW, "{:.2f} < {:.2f}".format(lts__high_aperture_utilization_threshold, lts__t_sectors_srcunit_tex_aperture_lookup_miss_ratio))
+            msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION, message, "{} Memory Usage".format(aperture_info))
+
+            speedup_type, speedup_value = get_estimated_speedup(action, aperture)
+            fe.speedup(msg_id, speedup_type, speedup_value)
+
+            fe.focus_metric(msg_id, metric_name, lts__t_sectors_srcunit_tex_aperture_lookup_miss, NvRules.IFrontend.Severity_SEVERITY_DEFAULT, "Decrease the lookup misses to {} memory".format(aperture_info.lower()))
