@@ -1,4 +1,4 @@
-# Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2023, NVIDIA CORPORATION. All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
 # are met:
@@ -92,21 +92,34 @@ def apply(handle):
             message += "but the address pattern, possibly caused by the stride between threads, results in {:.1f} sectors per request, or {:.1f}*32 = {:.1f} bytes of cache data transfers per request. ".format(sectors_per_request,sectors_per_request,32 * sectors_per_request)
             message += "The optimal thread address pattern for {:.1f} byte accesses would result in {:.1f}*32 = {:.1f} bytes of cache data transfers per request, to maximize L1TEX cache performance. ".format(smspAvgMemoryBytesPerInst,smspAvgMemoryBytesPerInst,32 * smspAvgMemoryBytesPerInst)
             message += "Check the @section:SourceCounters:Source Counters@ section for uncoalesced {}s.".format(access_info.lower())
-            msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_WARNING, message, "L1TEX {} Access Pattern".format(access_info))
-            fe.focus_metric(msg_id, "Sectors per L1TEX Request", sectors_per_request, NvRules.IFrontend.Severity_SEVERITY_HIGH if sectors_per_request > 2 * smspAvgMemoryBytesPerInst else NvRules.IFrontend.Severity_SEVERITY_LOW, "{:,.0f} / {:,.0f} > {:.1f}".format(sectors, requests, smspAvgMemoryBytesPerInst))
+            msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION, message, "L1TEX {} Access Pattern".format(access_info))
+
+            l1_bandwidth_name = "l1tex__t_sectors.sum.pct_of_peak_sustained_elapsed"
+            if l1_bandwidth_name in action:
+                l1_bandwidth_percent = action[l1_bandwidth_name].value()
+            else:
+                l1_bandwidth_percent = 0
+            improvement_percent = ((sectors_per_request - smspAvgMemoryBytesPerInst) / sectors_per_request) * (l1_bandwidth_percent / 100) * 100
+            fe.speedup(msg_id, NvRules.IFrontend.SpeedupType_LOCAL, improvement_percent)
+
+            if l1_bandwidth_percent != 0:
+                fe.focus_metric(msg_id, l1_bandwidth_name, l1_bandwidth_percent, NvRules.IFrontend.Severity_SEVERITY_LOW,
+                    "The higher the L1TEX peak utilization the more severe the issue becomes")
+            fe.focus_metric(msg_id, "Sectors per L1TEX Request", sectors_per_request, NvRules.IFrontend.Severity_SEVERITY_HIGH,
+                "Decrease the number of sectors per L1TEX request towards the minimal value ({:.1f})".format(smspAvgMemoryBytesPerInst))
 
     # L2 ==============================================================================================================
     l2_access_types = {
-        "tex_op_read" : (
+        "op_read" : (
             "Load"),
-        "tex_op_write" : (
+        "op_write" : (
             "Store"),
     }
 
     for access_type in l2_access_types:
         access_info = l2_access_types[access_type]
-        sectors = action.metric_by_name("lts__t_sectors_srcunit_{}.sum".format(access_type)).as_double()
-        requests = action.metric_by_name("lts__t_requests_srcunit_{}.sum".format(access_type)).as_double()
+        sectors = action.metric_by_name("lts__t_sectors_srcunit_tex_{}.sum".format(access_type)).as_double()
+        requests = action.metric_by_name("lts__t_requests_srcunit_tex_{}.sum".format(access_type)).as_double()
         sectors_per_request = sectors / requests if requests > 0 else 0
 
         # Anything less than 4 is not ideal, but we don't want to show a warning if it's very close.
@@ -115,8 +128,21 @@ def apply(handle):
             message += "The granularity of an L1TEX request to L2 is a 128 byte cache line. That is 4 consecutive 32-byte sectors per L2 request. "
             message += "However, this kernel only accesses an average of {:.1f} sectors out of the possible 4 sectors per cache line. ".format(sectors_per_request)
             message += "Check the @section:SourceCounters:Source Counters@ section for uncoalesced {}s and try to minimize how many cache lines need to be accessed per memory request.".format(access_info.lower())
-            msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_WARNING, message, "L2 {} Access Pattern".format(access_info))
-            fe.focus_metric(msg_id, "Sectors per L2 Request", sectors_per_request, NvRules.IFrontend.Severity_SEVERITY_HIGH if sectors_per_request <= 2 else NvRules.IFrontend.Severity_SEVERITY_LOW, "{:,.0f} / {:,.0f} < 4".format(sectors, requests))
+            msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION, message, "L2 {} Access Pattern".format(access_info))
+
+            l2_bandwidth_name = "lts__t_sectors_{}.sum.pct_of_peak_sustained_elapsed".format(access_type)
+            if l2_bandwidth_name in action:
+                l2_bandwidth_percent = action[l2_bandwidth_name].value()
+            else:
+                l2_bandwidth_percent = 0
+            improvement_percent = ((4 - sectors_per_request) / 4) * (l2_bandwidth_percent / 100) * 100
+            fe.speedup(msg_id, NvRules.IFrontend.SpeedupType_LOCAL, improvement_percent)
+
+            if l2_bandwidth_percent != 0:
+                fe.focus_metric(msg_id, l2_bandwidth_name, l2_bandwidth_percent, NvRules.IFrontend.Severity_SEVERITY_LOW,
+                    "The higher the L2 peak utilization the more severe the issue becomes")
+            fe.focus_metric(msg_id, "Sectors per L2 Request", sectors_per_request, NvRules.IFrontend.Severity_SEVERITY_HIGH,
+                "Increase the number of sectors used per L2 request towards the ideal value (4)")
 
     # DRAM ============================================================================================================
     if (True
@@ -130,10 +156,16 @@ def apply(handle):
         lts__read_sectors_not_hit = lts__read_sectors - lts__read_sectors_hits
 
         if dram__read_peak_pct > 50 and lts__read_sectors_not_hit < lts__read_sectors_misses:
-            message = "The memory access pattern for loads from device memory causes {:,.0f} sectors to be read from DRAM, which is {:.1f}x of the {:,.0f} sectors causing a miss in the L2 cache. ".format(lts__read_sectors_misses, lts__read_sectors_misses/lts__read_sectors_not_hit, lts__read_sectors_not_hit)
+            message = "The memory access pattern for loads from device memory causes {:,.0f} sectors to be read from DRAM, which is {:.1f}x of the {:,.0f} sectors which cause a miss in the L2 cache. ".format(lts__read_sectors_misses, lts__read_sectors_misses/lts__read_sectors_not_hit, lts__read_sectors_not_hit)
             message += "The DRAM fetch granularity for read misses in L2 is 64 bytes, i.e. the lower or upper half of an L2 cache line. "
             message += "Try changing your access pattern to make use of both sectors returned by a DRAM read request for optimal usage of the DRAM throughput. "
             message += "For strided memory reads, avoid strides of 64 bytes or larger to avoid moving unused sectors from DRAM to L2. "
-            msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_WARNING, message, "DRAM Excessive Read Sectors")
-            fe.focus_metric(msg_id, "DRAM Read Peak Utilization", dram__read_peak_pct, NvRules.IFrontend.Severity_SEVERITY_HIGH if dram__read_peak_pct > 75 else NvRules.IFrontend.Severity_SEVERITY_LOW, "{:.1f}% > 50%".format(dram__read_peak_pct))
-            fe.focus_metric(msg_id, "DRAM Excessive Read Sectors", lts__read_sectors_misses - lts__read_sectors, NvRules.IFrontend.Severity_SEVERITY_HIGH, "{:,.0f} > {:,.0f}".format(lts__read_sectors_misses, lts__read_sectors))
+            msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION, message, "DRAM Excessive Read Sectors")
+
+            improvement_percent = (1 - (lts__read_sectors - lts__read_sectors_not_hit) / lts__read_sectors) * (dram__read_peak_pct / 100) * 100
+            fe.speedup(msg_id, NvRules.IFrontend.SpeedupType_LOCAL, improvement_percent)
+
+            fe.focus_metric(msg_id, "dram__bytes_read.sum.pct_of_peak_sustained_elapsed", dram__read_peak_pct, NvRules.IFrontend.Severity_SEVERITY_LOW,
+                "The higher the DRAM peak read utilization the more severe the issue becomes")
+            fe.focus_metric(msg_id, "lts__t_sectors_srcunit_tex_op_read_lookup_hit.sum", lts__read_sectors_hits, NvRules.IFrontend.Severity_SEVERITY_HIGH,
+                "Increase the number of L2 read sector hits towards all read sectors ({:,.0f})".format(lts__read_sectors))
